@@ -12,8 +12,6 @@
 # under the License.
 
 """SQLAlchemy storage backend."""
-
-from __future__ import absolute_import
 import collections
 import datetime
 
@@ -21,6 +19,7 @@ from oslo_db import exception as dbexc
 from oslo_db.sqlalchemy import session as db_session
 from oslo_db.sqlalchemy import utils as oslo_sql_utils
 from oslo_log import log
+from oslo_utils import importutils
 from oslo_utils import timeutils
 import sqlalchemy as sa
 from sqlalchemy.engine import url as sqlalchemy_url
@@ -34,6 +33,8 @@ from panko import utils
 
 LOG = log.getLogger(__name__)
 
+osprofiler_sqlalchemy = importutils.try_import('osprofiler.sqlalchemy')
+
 
 AVAILABLE_CAPABILITIES = {
     'events': {'query': {'simple': True}},
@@ -136,6 +137,10 @@ class Connection(base.Connection):
             options.pop(opt.name, None)
         self._engine_facade = db_session.EngineFacade(self.dress_url(url),
                                                       **options)
+        if osprofiler_sqlalchemy:
+            osprofiler_sqlalchemy.add_tracing(sa,
+                                              self._engine_facade.get_engine(),
+                                              'db')
 
     @staticmethod
     def dress_url(url):
@@ -455,12 +460,13 @@ class Connection(base.Connection):
                                            dtype=dtype,
                                            value=v)
 
-    def clear_expired_data(self, ttl):
+    def clear_expired_data(self, ttl, max_count):
         """Clear expired data from the backend storage system.
 
         Clearing occurs according to the time-to-live.
 
         :param ttl: Number of seconds to keep records for.
+        :param max_count: Number of records to delete.
         """
         session = self._engine_facade.get_session()
         with session.begin():
@@ -468,17 +474,23 @@ class Connection(base.Connection):
             event_q = (session.query(models.Event.id)
                        .filter(models.Event.generated < end))
 
-            event_subq = event_q.subquery()
+            # NOTE(e0ne): it's not an optiomal from the performance point of
+            # view but it works with all databases.
+            ids = [i[0] for i in event_q.limit(max_count)]
             for trait_model in [models.TraitText, models.TraitInt,
                                 models.TraitFloat, models.TraitDatetime]:
-                (session.query(trait_model)
-                 .filter(trait_model.event_id.in_(event_subq))
-                 .delete(synchronize_session="fetch"))
-            event_rows = event_q.delete()
+                session.query(trait_model).filter(
+                    trait_model.event_id.in_(ids)
+                ).delete(synchronize_session="fetch")
+            event_rows = session.query(models.Event).filter(
+                models.Event.id.in_(ids)
+            ).delete(synchronize_session="fetch")
 
             # remove EventType and TraitType with no corresponding
-            # matching events and traits
+            # matching events
             (session.query(models.EventType)
              .filter(~models.EventType.events.any())
              .delete(synchronize_session="fetch"))
             LOG.info("%d events are removed from database", event_rows)
+
+            return event_rows
